library square_percent_indicater;

import 'dart:math';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

class SquarePercentIndicator extends StatelessWidget {
  final double width;
  final double height;
  final double progress;
  final double borderRadius;
  final Color progressColor;
  final Color shadowColor;
  final double progressWidth;
  final double shadowWidth;
  final Widget child;
  final bool reverse;

  const SquarePercentIndicator(
      {Key key,
        this.progress = 0.0,
        this.reverse = false,

        this.borderRadius = 5,
        this.progressColor = Colors.blue,
        this.shadowColor = Colors.grey,
        this.progressWidth = 5,
        this.shadowWidth = 5, this.child, this.width=150, this.height=150})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(borderRadius), ),
      width: width,
      height: height,
      child:reverse?

      Transform(
        alignment: Alignment.center,
        transform: Matrix4.rotationZ(pi/2),
        child: Transform(
          alignment: Alignment.center,
          transform: Matrix4.rotationY(pi),
          child: CustomPaint(
            child: Transform(
              alignment: Alignment.center,
              transform: Matrix4.rotationZ(-3*pi/2),
              child: Transform(
                alignment: Alignment.center,
                transform: Matrix4.rotationY(-pi),
                child: child??Container(),
              ),
            ),
            painter: RadialPainter(
                progress: progress,
                color: progressColor,
                shadowColor: shadowColor,
                reverse: reverse,
                strokeCap: StrokeCap.round,
                paintingStyle: PaintingStyle.stroke,
                strokeWidth: progressWidth,
                shadowWidth: shadowWidth,
                borderRadius: borderRadius),
          ),
        ),
      ):CustomPaint(
        child: child??Container(),
        painter: RadialPainter(
            progress: progress,
            color: progressColor,
            shadowColor: shadowColor,
            reverse: reverse,
            strokeCap: StrokeCap.round,
            paintingStyle: PaintingStyle.stroke,
            strokeWidth: progressWidth,
            shadowWidth: shadowWidth,
            borderRadius: borderRadius),
      ),
    );
  }
}

class RadialPainter extends CustomPainter {
  final double progress;
  final Color color;
  final Color shadowColor;
  final StrokeCap strokeCap;
  final PaintingStyle paintingStyle;
  final double strokeWidth;
  final double shadowWidth;
  final double borderRadius;
  final bool reverse;

  RadialPainter({
    this.progress,
    this.color,
    this.shadowColor,
    this.strokeWidth,
    this.shadowWidth,
    this.reverse,
    this.strokeCap,
    this.paintingStyle,
    this.borderRadius = 10,
  });

  @override
  void paint(Canvas canvas, Size size) {
    Paint paint = Paint()
      ..strokeWidth = strokeWidth
      ..color = color
      ..style = paintingStyle
      ..strokeCap = strokeCap;

    Paint shadowPaint = Paint()
      ..strokeWidth = shadowWidth
      ..color = shadowColor
      ..style = paintingStyle
      ..strokeCap = strokeCap;

    var path = Path();

    var shadowPath = Path();


    double borderRadiusPercent = (borderRadius / size.width) / 4;
    double startBorderPercent = 0.25 - borderRadiusPercent;
    double topLeftProgress = (progress - startBorderPercent) / (2 * borderRadiusPercent);
    topLeftProgress = max(min(topLeftProgress, 1), 0);
    double bottomLeftProgress = (progress - 0.25 - startBorderPercent) / (2 * borderRadiusPercent);
    bottomLeftProgress = max(min(bottomLeftProgress, 1), 0);
    double topRightProgress = (progress - 0.75 - startBorderPercent) / (borderRadiusPercent);
    topRightProgress = max(min(topRightProgress, 1), 0);
    double bottomRightProgress = (progress - 0.50 - startBorderPercent) / (2 * borderRadiusPercent);
    bottomRightProgress = max(min(bottomRightProgress, 1), 0);
    double topProgress = progress > (0.25 - borderRadiusPercent) ? 1 : (progress / (0.25 - borderRadiusPercent));
    double leftProgress = progress > (0.50 - borderRadiusPercent)
        ? 1
        : ((progress - 0.25 - 1 * borderRadiusPercent) / (0.25 - 2 * borderRadiusPercent));
    double bottomProgress = progress > (0.75 - borderRadiusPercent)
        ? 1
        : ((progress - 0.50 - 1 * borderRadiusPercent) / (0.25 - 2 * borderRadiusPercent));
    double rightProgress = progress == 1 ? 1 : ((progress - 0.75 - 1 * borderRadiusPercent) / (0.25 - 2 * borderRadiusPercent));
    drawTop(path, size, topProgress, borderRadiusPercent);
    drawLeft(path, size, leftProgress, borderRadiusPercent);
    drawBottom(path, size, bottomProgress, borderRadiusPercent);
    drawRight(path, size, rightProgress, borderRadiusPercent);
    drawTopLeft(path, size, topLeftProgress);
    drawBottomLeft(path, size, bottomLeftProgress);
    drawTopRight(path, size, topRightProgress);
    drawBottomRight(path, size, bottomRightProgress);


    drawTop(shadowPath, size, 1, borderRadiusPercent);
    drawLeft(shadowPath, size, 1, borderRadiusPercent);
    drawBottom(shadowPath, size, 1, borderRadiusPercent);
    drawRight(shadowPath, size, 1, borderRadiusPercent);
    drawTopLeft(shadowPath, size, 1);
    drawBottomLeft(shadowPath, size, 1);
    drawTopRight(shadowPath, size, 1);
    drawBottomRight(shadowPath, size, 1);

    canvas.drawPath(shadowPath, shadowPaint);
    canvas.drawPath(path, paint);

  }

  drawTop(Path path, Size size, double percent, double borderPercent) {
    double p = min(max(percent, 0), 1);
    path.moveTo(size.width - borderRadius, 0);
    if (p > 0) {
      path.lineTo(size.width - borderRadius - (size.width - 2 * borderRadius) * p, 0);
    }
  }

  drawLeft(Path path, Size size, double percent, double borderPercent) {
    double p = min(max(percent, 0), 1);
    path.moveTo(0, borderRadius);
    if (p > 0) {
      path.lineTo(0, borderRadius + (size.height - 2 * borderRadius) * p);
    }
  }

  drawBottom(Path path, Size size, double percent, double borderPercents) {
    double p = min(max(percent, 0), 1);
    path.moveTo(borderRadius, size.height);
    if (p > 0) {
      path.lineTo(borderRadius + (size.height - 2 * borderRadius) * p, size.height);
    }
  }

  drawRight(Path path, Size size, double percent, double borderPercent) {
    double p = min(max(percent, 0), 1);
    path.moveTo(size.width, size.height - borderRadius);
    if (p > 0) {
      path.lineTo(size.width, size.height - borderRadius - (size.height - 2 * borderRadius) * p);
    }
  }

  drawTopLeft(Path path, Size size, double progress) {
    path.addArc(Rect.fromCircle(center: Offset(borderRadius, borderRadius), radius: borderRadius), -pi / 2, -pi / 2 * progress);
  }

  drawBottomLeft(Path path, Size size, double progress) {
    path.addArc(
        Rect.fromCircle(center: Offset(borderRadius, size.height - borderRadius), radius: borderRadius), -pi, -pi / 2 * progress);
  }

  drawTopRight(Path path, Size size, double progress) {
    path.addArc(
        Rect.fromCircle(center: Offset(size.width - borderRadius, borderRadius), radius: borderRadius), 0, -pi / 2 * progress);
  }

  drawBottomRight(Path path, Size size, double progress) {
    path.addArc(Rect.fromCircle(center: Offset(size.width - borderRadius, size.height - borderRadius), radius: borderRadius),
        -1.5 * pi, -pi / 2 * progress);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return true;
  }
}
